@@ -22,9 +22,19 @@ import glob
 import imp
 import os
 
-import ansible.constants as C
+import six
+
+from ansible import constants
 from ansible.errors import AnsibleError
-from ansible.module_utils.splitter import split_args
+
+try:
+    # Try to import the Ansible 2 module first, it's the future-proof one
+    from ansible.parsing.splitter import split_args
+
+except ImportError:
+    # Fallback on the Ansible 1.9 module
+    from ansible.module_utils.splitter import split_args
+
 import yaml
 from yaml.composer import Composer
 from yaml.constructor import Constructor
@@ -69,7 +79,7 @@ VALID_KEYS = [
     'sudo_pass', 'when', 'connection', 'environment', 'args',
     'any_errors_fatal', 'changed_when', 'failed_when', 'always_run', 'delay', 'retries', 'until',
     'su', 'su_user', 'su_pass', 'no_log', 'run_once',
-    'become', 'become_user', 'become_method',
+    'become', 'become_user', 'become_method', FILENAME_KEY,
 ]
 
 BLOCK_NAME_TO_ACTION_TYPE_MAP = {
@@ -139,7 +149,7 @@ def find_children(playbook, playbook_dir):
     else:
         try:
             playbook_ds = parse_yaml_from_file(playbook[0])
-        except AnsibleError, e:
+        except AnsibleError as e:
             raise SystemExit(str(e))
     results = []
     basedir = os.path.dirname(playbook[0])
@@ -178,6 +188,7 @@ def play_children(basedir, item, parent_type, playbook_dir):
         'tasks': _taskshandlers_children,
         'pre_tasks': _taskshandlers_children,
         'post_tasks': _taskshandlers_children,
+        'block': _taskshandlers_children,
         'include': _include_children,
         'roles': _roles_children,
         'dependencies': _roles_children,
@@ -208,20 +219,39 @@ def _include_children(basedir, k, v, parent_type):
 
 
 def _taskshandlers_children(basedir, k, v, parent_type):
-    return [{'path': path_dwim(basedir, th['include']),
-             'type': 'tasks'}
-            for th in v if 'include' in th]
+    results = []
+    for th in v:
+        if 'include' in th:
+            # taskshandlers_children only get's called for playbooks, thus the
+            # actual type of the included tasks is the section containing the
+            # include, e.g. tasks, pre_tasks, or handlers.
+            assert(parent_type == 'playbook')
+            playbook_section = k
+            results.append({
+                'path': path_dwim(basedir, th['include']),
+                'type': playbook_section
+            })
+        elif 'block' in th:
+            results.extend(_taskshandlers_children(basedir, k, th['block'], parent_type))
+            if 'rescue' in th:
+                results.extend(_taskshandlers_children(basedir, k, th['rescue'], parent_type))
+            if 'always' in th:
+                results.extend(_taskshandlers_children(basedir, k, th['always'], parent_type))
+
+    return results
 
 
 def _roles_children(basedir, k, v, parent_type):
     results = []
     for role in v:
         if isinstance(role, dict):
-            if 'role' in role:
+            if 'role' in role or 'name' in role:
                 if 'tags' not in role or 'skip_ansible_lint' not in role['tags']:
-                    results.extend(_look_for_role_files(basedir, role['role']))
+                    results.extend(_look_for_role_files(basedir,
+                                                        role.get('role', role.get('name'))))
             else:
-                raise SystemExit('role dict {0} does not contain a "role" key'.format(role))
+                raise SystemExit('role dict {0} does not contain a "role" '
+                                 'or "name" key'.format(role))
         else:
             results.extend(_look_for_role_files(basedir, role))
     return results
@@ -246,9 +276,9 @@ def _rolepath(basedir, role):
         path_dwim(basedir, os.path.join('..', '..', role))
     ]
 
-    if C.DEFAULT_ROLES_PATH:
-        search_locations = C.DEFAULT_ROLES_PATH
-        if isinstance(search_locations, basestring):
+    if constants.DEFAULT_ROLES_PATH:
+        search_locations = constants.DEFAULT_ROLES_PATH
+        if isinstance(search_locations, six.string_types):
             search_locations = search_locations.split(os.pathsep)
         for loc in search_locations:
             loc = os.path.expanduser(loc)
@@ -353,7 +383,7 @@ def normalize_task_v1(task):
             else:
                 result[k] = v
         else:
-            if isinstance(v, basestring):
+            if isinstance(v, six.string_types):
                 v = _kv_to_dict(k + ' ' + v)
             elif not v:
                 v = dict(__ansible_module__=k)
@@ -417,7 +447,7 @@ def extract_from_list(blocks, candidates):
     results = list()
     for block in blocks:
         for candidate in candidates:
-            if candidate in block:
+            if isinstance(block, dict) and candidate in block:
                 if isinstance(block[candidate], list):
                     results.extend(add_action_type(block[candidate], candidate))
                 elif block[candidate] is not None:
@@ -472,6 +502,7 @@ def parse_yaml_linenumbers(data, filename):
     def construct_mapping(node, deep=False):
         mapping = Constructor.construct_mapping(loader, node, deep=deep)
         mapping[LINE_NUMBER_KEY] = node.__line__
+        mapping[FILENAME_KEY] = filename
         return mapping
 
     try:
